@@ -1286,9 +1286,9 @@ find_by_name( KBNODE keyblock, PKT_public_key *pk, const char *name,
 		return kk;
 	    }
 	    else if( is_RSA(pk->pubkey_algo) )
-		log_error("RSA key cannot be used in this version\n");
+		log_error(_("RSA key cannot be used in this version\n"));
 	    else
-		log_error("No key for userid\n");
+		log_error(_("No key for user-id\n"));
 	}
     }
     return NULL;
@@ -1325,9 +1325,9 @@ find_by_name_sk( KBNODE keyblock, PKT_secret_key *sk, const char *name,
 		return kk;
 	    }
 	    else if( is_RSA(sk->pubkey_algo) )
-		log_error("RSA key cannot be used in this version\n");
+		log_error(_("RSA key cannot be used in this version\n"));
 	    else
-		log_error("No key for userid\n");
+		log_error(_("No key for user-id\n"));
 	}
     }
     return NULL;
@@ -1366,7 +1366,7 @@ find_by_keyid( KBNODE keyblock, PKT_public_key *pk, u32 *keyid, int mode )
 		if( kk )
 		    cache_user_id( kk->pkt->pkt.user_id, aki );
 		else
-		    log_error("No userid for key\n");
+		    log_error(_("No user-id for key\n"));
 		return k; /* found */
 	    }
 	}
@@ -1406,7 +1406,7 @@ find_by_keyid_sk( KBNODE keyblock, PKT_secret_key *sk, u32 *keyid, int mode )
 		if( kk )
 		    cache_user_id( kk->pkt->pkt.user_id, aki );
 		else
-		    log_error("No userid for key\n");
+		    log_error(_("No user-id for key\n"));
 		return k; /* found */
 	    }
 	}
