@@ -58,7 +58,6 @@ typedef struct {
 
 
 static int underflow(IOBUF a);
-static const char *get_real_fname( IOBUF a );
 
 /****************
  * Read data from a file into buf which has an allocated length of *LEN.
@@ -89,33 +88,34 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
     FILE *fp = a->fp;
     size_t size = *ret_len;
     size_t nbytes = 0;
-    int c, rc = 0;
-    char *p;
+    int rc = 0;
 
     if( control == IOBUFCTRL_UNDERFLOW ) {
 	assert( size ); /* need a buffer */
-	for(; size; size-- ) {
-	    if( (c=getc(fp)) == EOF ) {
-		if( ferror(fp) && errno != EPIPE  ) {
-		    log_error("%s: read error: %s\n",
-					a->fname, strerror(errno));
-		    rc = G10ERR_READ_FILE;
-		}
-		else if( !nbytes )
-		    rc = -1; /* okay: we can return EOF now. */
-		break;
+	if ( feof(fp)) {	/* On terminals you could easiely read as many EOFs as you call 	*/
+	    rc = -1;		/* fread() or fgetc() repeatly. Every call will block until you press	*/
+	    *ret_len = 0;	/* CTRL-D. So we catch this case before we call fread() again.		*/
+	}
+	else {
+	    clearerr( fp );
+	    nbytes = fread( buf, 1, size, fp );
+	    if( feof(fp) && !nbytes )
+		rc = -1; /* okay: we can return EOF now. */
+	    else if( ferror(fp) && errno != EPIPE  ) {
+		log_error("%s: read error: %s\n",
+			  a->fname, strerror(errno));
+		rc = G10ERR_READ_FILE;
 	    }
-	    buf[nbytes++] = c & 0xff;
+	    *ret_len = nbytes;
 	}
-	*ret_len = nbytes;
     }
     else if( control == IOBUFCTRL_FLUSH ) {
-	for(p=buf; nbytes < size; nbytes++, p++ ) {
-	    if( putc(*p, fp) == EOF ) {
-		log_error("%s: write error: %s\n",
-				    a->fname, strerror(errno));
+	if( size ) {
+	    clearerr( fp );
+	    nbytes = fwrite( buf, 1, size, fp );
+	    if( ferror(fp) ) {
+		log_error("%s: write error: %s\n", a->fname, strerror(errno));
 		rc = G10ERR_WRITE_FILE;
-		break;
 	    }
 	}
 	*ret_len = nbytes;
@@ -149,7 +149,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 {
     block_filter_ctx_t *a = opaque;
     size_t size = *ret_len;
-    int c, rc = 0;
+    int c, needed, rc = 0;
     char *p;
 
     if( control == IOBUFCTRL_UNDERFLOW ) {
@@ -239,15 +239,20 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		}
 	    }
 
-	    for(; !rc && size && a->size; size--, a->size-- ) {
-		if( (c=iobuf_get(chain)) == -1 ) {
+	    while( !rc && size && a->size ) {
+		needed = size < a->size ? size : a->size;
+		c = iobuf_read( chain, p, needed );
+		if( c < needed ) {
+		    if( c == -1 ) c = 0;
 		    log_error("block_filter %p: read error (size=%lu,a->size=%lu)\n",
-				a,  (ulong)size, (ulong)a->size);
+			      a,  (ulong)size+c, (ulong)a->size+c);
 		    rc = G10ERR_READ_FILE;
 		}
 		else {
-		    *p++ = c;
-		    n++;
+		    size -= c;
+		    a->size -= c;
+		    p += c;
+		    n += c;
 		}
 	    }
 	}
@@ -420,7 +425,7 @@ print_chain( IOBUF a )
 
 	log_debug("iobuf chain: %d.%d `%s' filter_eof=%d start=%d len=%d\n",
 		   a->no, a->subno, desc, a->filter_eof,
-		   a->d.start, a->d.len );
+		   (int)a->d.start, (int)a->d.len );
     }
 }
 
@@ -449,6 +454,7 @@ iobuf_alloc(int use, size_t bufsize)
     a->no = ++number;
     a->subno = 0;
     a->opaque = NULL;
+    a->real_fname = NULL;
     return a;
 }
 
@@ -462,6 +468,7 @@ iobuf_close( IOBUF a )
 
     if( a && a->directfp ) {
 	fclose( a->directfp );
+	m_free( a->real_fname );
 	if( DBG_IOBUF )
 	    log_debug("iobuf_close -> %p\n", a->directfp );
 	return 0;
@@ -477,6 +484,7 @@ iobuf_close( IOBUF a )
 	if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE,
 					 a->chain, NULL, &dummy_len)) )
 	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
+	m_free(a->real_fname);
 	m_free(a->d.buf);
 	m_free(a);
     }
@@ -489,7 +497,7 @@ iobuf_cancel( IOBUF a )
     const char *s;
 
     if( a && a->use == 2 ) {
-	s = get_real_fname(a);
+	s = iobuf_get_real_fname(a);
 	if( s && *s )
 	    remove(s);	/* remove the file. Fixme: this will fail for MSDOZE*/
     }			/* because the file is still open */
@@ -550,6 +558,8 @@ iobuf_open( const char *fname )
     fcx->fp = fp;
     fcx->print_only_name = print_only;
     strcpy(fcx->fname, fname );
+    if( !print_only )
+	a->real_fname = m_strdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -614,6 +624,8 @@ iobuf_create( const char *fname )
     fcx->fp = fp;
     fcx->print_only_name = print_only;
     strcpy(fcx->fname, fname );
+    if( !print_only )
+	a->real_fname = m_strdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -644,6 +656,7 @@ iobuf_append( const char *fname )
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     strcpy(fcx->fname, fname );
+    a->real_fname = m_strdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -670,6 +683,7 @@ iobuf_openrw( const char *fname )
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     strcpy(fcx->fname, fname );
+    a->real_fname = m_strdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -703,6 +717,7 @@ iobuf_fopen( const char *fname, const char *mode )
 	return NULL;
     a = iobuf_alloc(1, 8192 );
     a->directfp = fp;
+    a->real_fname = m_strdup( fname );
 
     if( DBG_IOBUF )
 	log_debug("iobuf_fopen -> %p\n", a->directfp );
@@ -745,6 +760,10 @@ iobuf_push_filter2( IOBUF a,
      */
     b = m_alloc(sizeof *b);
     memcpy(b, a, sizeof *b );
+    /* fixme: it is stupid to keep a copy of the name at every level
+     * but we need the name somewhere because the name known by file_filter
+     * may have been released when we need the name of the file */
+    b->real_fname = a->real_fname? m_strdup(a->real_fname):NULL;
     /* remove the filter stuff from the new stream */
     a->filter = NULL;
     a->filter_ov = NULL;
@@ -811,6 +830,7 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	b = a->chain;
 	assert(b);
 	m_free(a->d.buf);
+	m_free(a->real_fname);
 	memcpy(a,b, sizeof *a);
 	m_free(b);
 	return 0;
@@ -847,6 +867,7 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	 */
 	b = a->chain;
 	m_free(a->d.buf);
+	m_free(a->real_fname);
 	memcpy(a,b, sizeof *a);
 	m_free(b);
 	if( DBG_IOBUF )
@@ -884,6 +905,7 @@ underflow(IOBUF a)
 		log_debug("iobuf-%d.%d: pop `%s' in underflow\n",
 					a->no, a->subno, a->desc );
 	    m_free(a->d.buf);
+	    m_free(a->real_fname);
 	    memcpy(a, b, sizeof *a);
 	    m_free(b);
 	    print_chain(a);
@@ -952,6 +974,7 @@ underflow(IOBUF a)
 					       a->no, a->subno, a->desc );
 		print_chain(a);
 		m_free(a->d.buf);
+		m_free(a->real_fname);
 		memcpy(a,b, sizeof *a);
 		m_free(b);
 		print_chain(a);
@@ -1058,28 +1081,40 @@ iobuf_read(IOBUF a, byte *buf, unsigned buflen )
 
     if( a->unget.buf || a->nlimit ) {
 	/* handle special cases */
-	for(n=0 ; n < buflen; n++, buf++ ) {
+	for(n=0 ; n < buflen; n++ ) {
 	    if( (c = iobuf_readbyte(a)) == -1 ) {
 		if( !n )
 		    return -1; /* eof */
 		break;
 	    }
 	    else
-		*buf = c;
+		if( buf ) *buf = c;
+	    if( buf ) buf++;
 	}
 	return n;
     }
 
     n = 0;
     do {
-	for( ; n < buflen && a->d.start < a->d.len; n++ )
-	    *buf++ = a->d.buf[a->d.start++];
+	if( n < buflen && a->d.start < a->d.len ) {
+	    unsigned size = a->d.len - a->d.start;
+	    if( size > buflen - n )
+		size = buflen - n;
+	    if( buf )
+		memcpy( buf, a->d.buf + a->d.start, size );
+	    n += size;
+	    a->d.start += size;
+	    if( buf )
+		buf += size;
+	}
 	if( n < buflen ) {
 	    if( (c=underflow(a)) == -1 ) {
 		a->nbytes += n;
 		return n? n : -1/*EOF*/;
 	    }
-	    *buf++ = c; n++;
+	    if( buf )
+		*buf++ = c;
+	    n++;
 	}
     } while( n < buflen );
     a->nbytes += n;
@@ -1140,8 +1175,14 @@ iobuf_write(IOBUF a, byte *buf, unsigned buflen )
 	BUG();
 
     do {
-	for( ; buflen && a->d.len < a->d.size; buflen--, buf++ )
-	    a->d.buf[a->d.len++] = *buf;
+	if( buflen && a->d.len < a->d.size ) {
+	    unsigned size = a->d.size - a->d.len;
+	    if( size > buflen ) size = buflen;
+	    memcpy( a->d.buf + a->d.len, buf, size );
+	    buflen -= size;
+	    buf += size;
+	    a->d.len += size;
+	}
 	if( buflen ) {
 	    if( iobuf_flush(a) )
 		return -1;
@@ -1237,6 +1278,7 @@ iobuf_get_filelength( IOBUF a )
 	return 0;
     }
 
+    /* Hmmm: file_filter may have already been removed */
     for( ; a; a = a->chain )
 	if( !a->chain && a->filter == file_filter ) {
 	    file_filter_ctx_t *b = a->filter_ov;
@@ -1317,9 +1359,13 @@ iobuf_seek( IOBUF a, ulong newpos )
 /****************
  * Retrieve the real filename
  */
-static const char *
-get_real_fname( IOBUF a )
+const char *
+iobuf_get_real_fname( IOBUF a )
 {
+    if( a->real_fname )
+	return a->real_fname;
+
+    /* the old solution */
     for( ; a; a = a->chain )
 	if( !a->chain && a->filter == file_filter ) {
 	    file_filter_ctx_t *b = a->filter_ov;
